<?php

namespace Planejamento\Logic\Cardapio;

use Geral\Entities\Filial;
use Geral\Entities\Itemreti;
use Geral\Entities\Retirada;
use Geral\Repositories\Servico;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Planejamento\Logic\Planejamento;
use Setforms2\Shared\Registry;
use Zend_Date;
use \Planejamento\Entities\Recepadr as Recepadr;
use Planejamento\Entities\Compcard;
use TkDateTime;
use Planejamento\Entities\Cardadia;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use Planejamento\Logic\Cardapio\Exception\Cardapio as CardapioException;
use Planejamento\Entities\Recefili as Recefili;
use \Planejamento\Entities\Receserv as Receserv;
use Geral\Entities\Paramgeral;
use Geral\Entities\Paramfilial;
use Planejamento\Entities\Paramplaneja;
use Symfony\Component\Config\Definition\Exception\ForbiddenOverwriteException;
use Planejamento\Logic\ConsumoPlanejadoRel\ConsumoPlanejadoRel;
use Planejamento\Repositories\Custpadr;
use Planejamento\Entities\Cardacli;
use Planejamento\Util\Parametros;

class Cardapio {
	const GANHO = 'Ganho';
	const PERDA = 'Perda';
	const EXPORTA_CARDAPIO_CONFIG = 'EXPORTA_CARDAPIO_CONFIG';
    const SOBREPOE_PRATO = '1';
    const NAO_SOBREPOE_PRATO = '2';
    const APENAS_COMENSAIS = '3';

	private $nrorg;
	
	/**
	 * @var $filial \Geral\Entities\Filial
	 */
	private $filial;
	
	/**
	 * @var $servico \Geral\Entities\Servico
	 */
	private $servico;
	
	/**
	 * @var $cardadia \Planejamento\Repositories\Cardadia
	 */
	private $cardadia;
	
	/**
	 * @var $cardadiaEntity \Planejamento\Entities\Cardadia
	 */
	private $cardadiaEntity;
	public function __construct($nrorg, $cdfilial, $cdservico) {
		$this->nrorg = $nrorg;
		$this->filial = \Geral\Entities\Filial::getObject ( array (
			"cdfilial" => $cdfilial,
			"nrorg" => $nrorg 
			) );
		$this->servico = \Geral\Entities\Servico::getObject ( array (
			"cdfilial" => $cdfilial,
			"cdservfili" => $cdservico,
			"nrorg" => $nrorg 
			) );
		$this->cardadia = \Planejamento\Entities\Cardadia::getRepository ();
	}

    public function copiaPratos(
        $dtcardapio, $dtprocessamento, $cdoperador, $nrseqpratcar, $nrordpratcar,
        $nrcardapioOrigem, $cdtipoprat, $tipoCopia, $montagem, $comensal = null
    ) {
        $manager = \Setforms2\Shared\Registry::getEntityManager ();

        $conn = $manager->getConnection ();
        $conn->beginTransaction ();

        try {
            $dias = $this->diasDisponiveisParaAlteracaoNoCardapio($dtcardapio, $dtprocessamento);

            // Dados do prato espelho.
            $cardadiaOrigem = $this->getCardapioPorNumeroDoCardapio ( $nrcardapioOrigem );
            /**
             * @var $pratoOrigemEntity \Planejamento\Entities\Compcard
             */
            $pratoOrigemEntity = $cardadiaOrigem->getComposicaoDoCardapio ( $nrseqpratcar );
            $cdfilauxprat = $pratoOrigemEntity->getCdfilauxprat ();
            $cdprato = $pratoOrigemEntity->getCdprato ();

            // Retira dia do prato espelho da lista de pratos a serem copiados.
            $diaPratoOrigem = new TkDateTime ( $cardadiaOrigem->getDtcardapio () );

            /**
             * @var $dia Zend_Date
             */
            foreach ( $dias as $key => $dia ) {
                if ($dia == $diaPratoOrigem) {
                    unset ( $dias [$key] );
                    continue;
                }

                $dtcardapio = $dia->get ( 'dd/MM/y' );
                /** @var \Planejamento\Entities\Cardadia $cardadia */
                $cardadia = $this->getCardapioPorData ( $dtcardapio );

                if (is_null($comensal[$dia->get('e')])) {
                    if ($cardadia !== false) {
                        $comensais = $cardadia->getQtcomesticar();
                    } else {
                        $diaSemana = $dia->get('EEE');
                        if ($diaSemana === 's�b') $diaSemana = 'sab';
                        $qtComensais = 'getQtcomesti' . $diaSemana;
                        $comensais = $this->servico->$qtComensais();
                    }
                } else {
                    $comensais = $comensal[$dia->get('e')];
                }

                if (! $cardadia) {
                    /**
                     * @var $cardadia \Planejamento\Entities\Cardadia
                     */
                    $cardadia = $this->criaCardapio ( $cdoperador, $dia->get('dd/MM/y'), $comensais);
                    if ($cardadia === null)
                        continue;
                }
                $vrpeconscard = $this->_getPercentualConsumo ( $comensais, $montagem, $cardadia );

                if ($tipoCopia === self::SOBREPOE_PRATO) {
                    $this->_copiaPratosSobrepondo(
                        $cardadia, $nrseqpratcar, $vrpeconscard,
                        $nrordpratcar, $cdfilauxprat, $cdprato, $cdtipoprat
                    );
                } else if ($tipoCopia === self::NAO_SOBREPOE_PRATO)  {
                    $this->_copiaPratosSemSobrepor(
                        $cardadia, $nrseqpratcar, $nrordpratcar, $cdfilauxprat,
                        $cdprato, $vrpeconscard, $cdtipoprat
                    );
                } else if ($tipoCopia === self::APENAS_COMENSAIS){
                    $this->_copiaApenasComensais(
                        $cardadia, $nrseqpratcar, $vrpeconscard
                    );
                }

                $manager->flush ();
            }
            $conn->commit ();
        } catch ( \Planejamento\Logic\Exception $e ) {
            $conn->rollBack ();
            throw $e;
        }
    }

	private function _copiaPratosSobrepondo(
        \Planejamento\Entities\Cardadia $cardadia, $nrseqpratcar, $vrpeconscard,
        $nrordpratcar, $cdfilauxprat, $cdprato, $cdtipoprat
    ) {
		try {
            $pratoLogic = new ComposicaoPrato ( $this->nrorg, $this->filial->getCdfilial (), $cardadia->getNrcardapio (), $nrseqpratcar );
            $pratoLogic->alteraComposicaoDoCardapio ( $vrpeconscard, $nrordpratcar, $cdfilauxprat, $cdprato, $cdtipoprat );
		} catch ( \Planejamento\Logic\Exception $e ) {
			throw $e;
		}
	}


	private function _copiaPratosSemSobrepor(
        \Planejamento\Entities\Cardadia $cardadia, $nrseqpratcar, $nrordpratcar,
        $cdfilauxprat, $cdprato, $vrpeconscard, $cdtipoprat
    ) {
		try {
            $pratoLogic = new ComposicaoPrato (
                $this->nrorg, $this->filial->getCdfilial(),
                $cardadia->getNrcardapio(), $nrseqpratcar
            );
            if (!$pratoLogic->getComposicaoCardapio()) {
                $pratoLogic->inserePratoNoCardapio(
                    $nrordpratcar, $cdfilauxprat, $cdprato, $vrpeconscard,
                    $cdtipoprat
                );
            }
        } catch ( \Planejamento\Logic\Exception $e ) {
			throw $e;
		}
	}

    private function _copiaApenasComensais(
        \Planejamento\Entities\Cardadia $cardadia, $nrseqpratcar, $vrpeconscard
    ) {
        try {
            $pratoLogic = new ComposicaoPrato (
                $this->nrorg, $this->filial->getCdfilial (),
                $cardadia->getNrcardapio (), $nrseqpratcar
            );
            $compcard = $pratoLogic->getComposicaoCardapio ();
            if ($compcard) {
                $compcard->setVrpeconscard(
                    (float)str_replace(',', '.', $vrpeconscard)
                );
                $compcard->persist ();
            }
        } catch ( \Planejamento\Logic\Exception $e ) {
            throw $e;
        }
    }

	/**
	 *
	 * @param
	 *        	$cdoperador
	 * @param
	 *        	$dtcardapio
	 * @param null $qtcomesticar        	
	 * @param null $qtcliesticar        	
	 * @param null $qtfunesticar        	
	 * @param null $qtdoaesticar        	
	 * @return bool null \Planejamento\Entities\Cardadia
	 * @throws Exception\RestricoesCardadia
	 */
	public function criaCardapio($cdoperador, $dtcardapio, $qtcomesticar = null, $qtcliesticar = null, $qtfunesticar = null, $qtdoaesticar = null) {
		$cardadia = $this->getCardapioPorData ( $dtcardapio );
		if ($this->getCardapioPorData ( $dtcardapio ))
			return $cardadia;
        if (
            \Windows\Pla00100\TabPla00100Elaboracaocardapio\Facade::getBloqueiaCardapioDiaNaoUtil()
            && $this->isFeriado($dtcardapio)
        ) {
            CardapioException::diaNaoUtil();
        }
		
		$manager = \Setforms2\Shared\Registry::getEntityManager ();
		
		$cardadia = new Cardadia ();
		$cardadia->setChavesPrimarias ( $this->nrorg, $this->filial->getCdfilial () );
		
		if (! $qtcomesticar) {
			$diaDaSemana = new \TkDateTime ( $dtcardapio );
			$diaDaSemana = $diaDaSemana->get ( 'EEE' );
			if ($diaDaSemana == 'sáb')
				$diaDaSemana = 'sab';
			$comest = 'getQtcomesti' . $diaDaSemana;
			$cliest = 'getQtcliesti' . $diaDaSemana;
			$funest = 'getQtfunesti' . $diaDaSemana;
			$doaest = 'getQtdoaesti' . $diaDaSemana;
			$qtcomesticar = $this->servico->$comest ();
			$qtcliesticar = $this->servico->$cliest ();
			$qtfunesticar = $this->servico->$funest ();
			$qtdoaesticar = $this->servico->$doaest ();
			if ($qtcomesticar == 0) {
				return null;
			}
		}
		
		$cardadia->setQuantidadeComensaisEstimadas ( $qtcomesticar, $qtcliesticar, $qtfunesticar, $qtdoaesticar );
		
		$dtcardapio = \Geral\Util\DateUtil::truncateData ( \Geral\Util\DateUtil::getDataDeString ( $dtcardapio ) );
		$dtmescard = \Geral\Util\DateUtil::truncateData ( \Geral\Util\DateUtil::getPrimeiroDiaDoMes ( $dtcardapio ) );
		
		$cardadia->setDtcardapio ( $dtcardapio );
		$cardadia->setCdservfili ( $this->servico->getCdservfili () );
		$cardadia->setCdoperador ( $cdoperador );
		$cardadia->setDtmescard ( $dtmescard );
		$cardadia->setQtcomesticar ( $qtcomesticar );
        $cardadia->setDtinclusao(DateUtil::getDataAtual());
        $cardadia->setDtultatu(DateUtil::getDataAtual());
        $cardadia->setCdoperinclusao(Ambiente::getCdOperador());
		
		$cardadia->persist ();
		$manager->flush ();
		return $cardadia;
	}
	public function inserePratoNoCardapio($nrorg, $cdfilial, $nrcardapio, $nrseqpratcar, $nrordpratcar, $cdfilauxprat, $cdprato, $vrpeconscard, $cdtipoprat, $idpratoespe = 'N') {
		$manager = \Setforms2\Shared\Registry::getEntityManager ();
		$compcard = new \Planejamento\Entities\Compcard ();
		
		// pk global
		$compcard->setNrorg ( ( int ) $nrorg );
		
		// Compcard e Cardadia pk
		$compcard->setCdfilial ( $cdfilial );
		$compcard->setNrcardapio ( $nrcardapio );
		
		// Compcard pk
		$compcard->setNrseqpratcar ( ( int ) $nrseqpratcar );
		
		$compcard->setCdfilauxprat ( $cdfilauxprat );
		$compcard->setCdprato ( $cdprato );
		$compcard->setCdtipoprat ( $cdtipoprat );
		$compcard->setNrordpratcar ( $nrordpratcar );
		$compcard->setVrpeconscard ( str_replace ( ',', '.', $vrpeconscard ) );
		$compcard->setIdpratoespe ( $idpratoespe );
		
		$compcard->persist ();
		$manager->flush ();
	}
	public function atualizaPratoNoCardapio(\Planejamento\Entities\Compcard $compcard, $cdfilauxprat, $cdprato, $vrpeconscard, $idpratoespe = 'N') {
		$manager = \Setforms2\Shared\Registry::getEntityManager ();
		
		// Prato pk
		$compcard->setCdfilauxprat ( $cdfilauxprat );
		$compcard->setCdprato ( $cdprato );
		
		$compcard->setVrpeconscard ( str_replace ( ',', '.', $vrpeconscard ) );
		$compcard->setIdpratoespe ( $idpratoespe );
		
		$compcard->persist ();
		$manager->flush ();
	}
	public function atualizaComensaisNoCardapio(
        $cdoperador, $dtcardapio, $qtcliesticar, $qtfunesticar, $qtdoaesticar, $qtcomesticar, $dscardapio = null
    ) {
		$manager = \Setforms2\Shared\Registry::getEntityManager ();
		$cardadia = $this->getCardapioPorData ( $dtcardapio );
		if (! $cardadia) {
			return $this->criaCardapio ( $cdoperador, $dtcardapio, $qtcomesticar, $qtcliesticar, $qtfunesticar, $qtdoaesticar );		
		}
		
		$cardadia->setQtcliesticar ( $qtcliesticar );
		$cardadia->setQtfunesticar ( $qtfunesticar );
		$cardadia->setQtdoaesticar ( $qtdoaesticar );
		$cardadia->setQtcomesticar ( $qtcomesticar );
		$cardadia->setDscardapio ( $dscardapio );
		
		$cardadia->persist ();
		$manager->flush ();
		
		return $cardadia;
	}
        
        
    public function atualizaComensaisCliente($nrcardapio, array $clientes, array $destinos, array $comensais, $cdoperador){
        try {
        	$manager = \Setforms2\Shared\Registry::getEntityManager();
            
            foreach($clientes as $indice => $cliente) {
                if (array_key_exists($indice, $destinos) === false) {
                    continue;
                }

            	$destino = str_pad($destinos[$indice], 3, 0, STR_PAD_LEFT);
                if ($destino == 000) continue;
                $dtAtual = \Geral\Util\DateUtil::getDataAtual();
                $cardapioCli = Cardacli::getObject(
                	array('cdfilial' => $this->filial->getCdfilial(), 'nrorg' => $this->nrorg, 
                          'nrcardapio' => $nrcardapio, 'cdcliente' => $cliente, 
                          'cddestfili' => $destino));

                if ($cardapioCli instanceof Cardacli) {
                	$cardapioCli->atualizaComensaisCli($this->nrorg, $comensais[$indice], $cdoperador, $dtAtual);
                } else {
                	$cardacli = new Cardacli();
                    $cardacli->insereCardacli(
                    	$this->nrorg, $this->filial->getCdfilial(), $nrcardapio, $cliente, 
                        $destino, $comensais[$indice], $cdoperador, $dtAtual);                    
                }
                    
                $manager->flush();
            }    
        } catch (\Exception $e) {
        	throw $e;
        }
    }
        
        
	public function deletaCardapio($dtcardapio) {
		try {
            // $this->_validaDataParaAlterarCardapio($dtcardapio, $dtinicial, $dtfinal, $dtprocessamento);

            /**
             * @var $cardapio Cardadia
             */
            $cardapio = self::getCardapioPorData($dtcardapio);
            if (!$cardapio instanceof Cardadia) {
                return;
            }

            $nrCardapio = $cardapio->getNrcardapio();

            /**
             * Esclui Pratos do cardapio
             */
            $manager = \Setforms2\Shared\Registry::getEntityManager();
            $manager->beginTransaction();

            $this->deletaPratosDoCardapio($nrCardapio);
            $manager->flush();

            /**
             * Exclui possíveis retiradas
             */
            self::deletaRetiradas(
                $this->nrorg, $this->filial->getCdfilial(), $nrCardapio
            );

            /**
             * Exclui o cardapio
             */
//            $connection = \Setforms2\Shared\Registry::getEntityManager();
//            $connection->beginTransaction();

            $cardapio->delete();
            $manager->flush();

            $manager->commit();
            return true;
        }catch (\Exception $e) {
            throw $e;
        }
	}
	public static function deletaRetiradas($nrorg, $filial, $nrCardapio) {
		try {
            /**
             * @var $retirada Retirada
             */
            $retirada = Retirada::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCARDAPIO = ?', array(
                $nrorg,
                $filial,
                $nrCardapio
            )
            );

            if ($retirada instanceof Retirada) {
                $manager = \Setforms2\Shared\Registry::getEntityManager();
                $manager->beginTransaction();

                $nrRetirada = $retirada->getNrretirada();
                $itensReti = Itemreti::getObjectsByWhere(
                    'NRORG = ? AND CDFILIAL = ? AND NRCARDAPIO = ? AND NRRETIRADA = ?',
                    array(
                        $nrorg,
                        $filial,
                        $nrCardapio,
                        $nrRetirada
                    )
                );

                /**
                 * @var $item Itemreti
                 */
                foreach ($itensReti as $item) {
                    $item->delete();
                    $manager->flush();
                }

                $retirada->delete();
                $manager->flush();

                $manager->commit();
            }
        } catch (\Exception $e) {
            throw $e;
        }
	}
	public function deletaPratosDoCardapio($nrCardapio) {
        try {
            // public function deletaPratosDoCardapioNoPeriodo($dtinicial, $dtfinal)
            $compscard = Compcard::getObjectsByWhere(
                'CDFILIAL = ? AND NRORG = ? AND NRCARDAPIO = ?', array(
                $this->filial->getCdfilial(),
                $this->nrorg,
                $nrCardapio
            )
            );

            /**
             * @var $compcard Compcard
             */
            foreach ($compscard as $compcard) {
                $compcard->delete();
            }
        } catch (\Exception $e) {
            throw $e;
        }
	}
	private function _validaDataParaAlterarCardapio($dtcardapio, $dtinicial, $dtfinal, $dtprocessamento) {
		$dtprocessamento = new \TkDateTime ( $dtprocessamento, 'dd/MM/y' );
		$dt1 = new \TkDateTime ( $dtinicial );
		$dt2 = new \TkDateTime ( $dtfinal );
		if ($dt1->daysAfter ( $dt2 ) < 0) {
			\Geral\Exceptions\ExcecoesData::dataFinalMenorQueDataInicial ();
		}
		
		$dtcardapio = new \Zend_Date ( $dtcardapio, 'MM/y' );
		/**
		 * @var $dtcardapio \Zend_Date
		 */
		if (($dt1->get ( 'MM/y' ) !== $dtcardapio->get ( 'MM/y' )) || $dt1->get ( 'MM/y' ) !== $dt2->get ( 'MM/y' )) {
			throw new \Geral\Exceptions\ExcecoesData ( 'msg_data_entrada_fora_periodo' );
		}
		$dias = $dtprocessamento->daysAfter ( $dtinicial );
		if ($dias <= 0) {
			throw new \Geral\Exceptions\ExcecoesData ( 'CARDAPIO_BLOQUEADO' );
		}
	}
	private function _getPercentualConsumo($comensais, $montagem, \Planejamento\Entities\Cardadia $cardapioDoDia) {
		if ($montagem === 'C') {
			$vrpeconscard = $comensais ? $comensais : 100;
		} else {
			$percentualComensais = ( float ) ($comensais);
			$total = $cardapioDoDia->getQtcomesticar ();
			$vrpeconscard = $percentualComensais ? ($percentualComensais / $total) * 100 : 100;
		}
		return $vrpeconscard;
	}
    /*private function _getPercentualConsumo($comensais, $diaDaSemana,\Planejamento\Entities\Cardadia $cardapioDoDia) {
        if (array_key_exists ( 'PERCENTUAL', $comensais )) {
            $percentualComensais = $comensais ['PERCENTUAL'] [$diaDaSemana];
            $vrpeconscard = $percentualComensais ?  : 100;
        } else {
            $percentualComensais = ( float ) ($comensais ['ABSOLUTO'] [$diaDaSemana]);
            $total = $cardapioDoDia->getQtcomesticar ();
            $vrpeconscard = $percentualComensais ? ($percentualComensais / $total) * 100 : 100;
        }
        return $vrpeconscard;
    }*/
	public static function getUltimoDiaMes($dataInicial) {
		$data = new \TkDateTime ( $dataInicial, 'dd/MM/y', 'pt_BR' );
		$qtdeDiasDoMes = cal_days_in_month ( CAL_GREGORIAN, $data->get ( 'MM' ), $data->get ( 'y' ) );
		$data->setDay ( $qtdeDiasDoMes );
		$dataFinal = $data->get ( 'dd/MM/y' );
		return $dataFinal;
	}
	
	/**
	 *
	 * @param
	 *        	$dtcardapio
	 * @return bool Planejamento\Entities\Cardadia
	 */
	public function getCardapioPorData($dtcardapio) {
		$cardadia = \Planejamento\Entities\Cardadia::getObjectByWhere ( 'NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ? AND DTCARDAPIO = ?', array (
			$this->nrorg,
			$this->filial->getCdfilial (),
			$this->servico->getCdservfili (),
			$dtcardapio 
			) );
		return $cardadia;
	}
	
	/**
	 *
	 * @param
	 *        	$nrcardapio
	 * @return object Planejamento\Entities\Cardadia
	 */
	public function getCardapioPorNumeroDoCardapio($nrcardapio) {
		$cardadia = Cardadia::getObject ( array (
			'nrorg' => $this->nrorg,
			'cdfilial' => $this->filial->getCdfilial (),
			'nrcardapio' => $nrcardapio 
			) );
		
		return $cardadia;
	}
	public function alteraComposicaoDoCardapio($nrcardapio, $vrpeconscard, $nrseqpratcar, $nrordpratcar, $cdfilauxprat, $cdprato, $cdtipoprat) {
		if (! $vrpeconscard) {
			CardapioException::informarComensais ();
		}
		
		$composicaoPrato = new ComposicaoPrato ( $this->nrorg, $this->filial->getCdfilial (), $nrcardapio, $nrseqpratcar );
		$composicaoPrato->alteraComposicaoDoCardapio ( $vrpeconscard, $nrordpratcar, $cdfilauxprat, $cdprato, $cdtipoprat );
	}
	public function colaCardapioAcrescentando($nrcardapioOrigem, $nrcardapioDestino) {
		$conn = \Setforms2\Shared\Registry::getEntityManager ()->getConnection ();
		$conn->beginTransaction ();
		
		try {
			$cardadiaOrig = $this->getCardapioPorNumeroDoCardapio ( $nrcardapioOrigem );
			$cardadiaDest = $this->getCardapioPorNumeroDoCardapio ( $nrcardapioDestino );
			$composicaoPratosOrig = $cardadiaOrig->getComposicoesDoCardapio ();
			
			$composicaoPrato = new ComposicaoPrato ( $this->nrorg, $this->filial->getCdfilial (), $nrcardapioDestino );
			
			/**
			 * @var $composicaoPratoOrig Compcard
			 */
			foreach ( $composicaoPratosOrig as $composicaoPratoOrig ) {
				$composicaoPratoDest = $cardadiaDest->getComposicaoDoCardapio ( $composicaoPratoOrig->getNrseqpratcar () );
				if ($composicaoPratoDest)
					continue;
				$composicaoPrato->inserePratoNoCardapio ( $composicaoPratoOrig->getNrordpratcar (), $composicaoPratoOrig->getCdfilauxprat (), $composicaoPratoOrig->getCdprato (), $composicaoPratoOrig->getVrpeconscard (), $composicaoPratoOrig->getCdtipoprat (), $composicaoPratoOrig->getNrseqpratcar () );
			}
			
			$conn->commit ();
		} catch ( CardapioException $e ) {
			tk_show_message_by_name ( $e->getMessageName () );
			$conn->rollBack ();
		} catch ( \Exception $e ) {
			tk_warning ( $e->getMessage () );
			$conn->rollBack ();
		}
	}
	public function colaCardapioSobrepondo($nrcardapioOrigem, $nrcardapioDestino) {
		$conn = \Setforms2\Shared\Registry::getEntityManager ()->getConnection ();
		$conn->beginTransaction ();
		
		try {
			$cardadiaOrig = $this->getCardapioPorNumeroDoCardapio ( $nrcardapioOrigem );
			$composicaoPratosOrig = $cardadiaOrig->getComposicoesDoCardapio ();
			
			/**
			 * @var $composicaoPratoOrig Compcard
			 */
			foreach ( $composicaoPratosOrig as $composicaoPratoOrig ) {
				$this->alteraComposicaoDoCardapio ( $nrcardapioDestino, $composicaoPratoOrig->getVrpeconscard (), $composicaoPratoOrig->getNrseqpratcar (), $composicaoPratoOrig->getNrordpratcar (), $composicaoPratoOrig->getCdfilauxprat (), $composicaoPratoOrig->getCdprato (), $composicaoPratoOrig->getCdtipoprat () );
			}
			
			$conn->commit ();
		} catch ( CardapioException $e ) {
			tk_show_message_by_name ( $e->getMessageName () );
			$conn->rollBack ();
		} catch ( \Exception $e ) {
			tk_warning ( $e->getMessage () );
			$conn->rollBack ();
		}
	}
	public function colaPratoNoCarcapio($nrCardapioOrigem, $sequencialOrigem, $nrCardapioDestino, $sequencialDestino, $ordenacaoDestino, $cdtipoprat) {
		$conn = \Setforms2\Shared\Registry::getEntityManager ()->getConnection ();
		$conn->beginTransaction ();
		
		$pratoLogic = new ComposicaoPrato ( $this->nrorg, $this->filial->getCdfilial (), $nrCardapioOrigem, $sequencialOrigem );
		$pratoEntityOrigem = $pratoLogic->getComposicaoCardapio ();
		if (! $pratoEntityOrigem) {
			throw new \Exception ( 'Prato inválido' );
		}
		
		try {
			$this->alteraComposicaoDoCardapio ( $nrCardapioDestino, $pratoEntityOrigem->getVrpeconscard (), $sequencialDestino, $ordenacaoDestino, $pratoEntityOrigem->getCdfilauxprat (), $pratoEntityOrigem->getCdprato (), $cdtipoprat );
		} catch ( \Exception $e ) {
			$conn->rollBack ();
			throw $e;
		}
		
		$conn->commit ();
		
		return array (
			'VRPECONSCARD' => $pratoEntityOrigem->getVrpeconscard (),
			'NMPRATO' => $pratoEntityOrigem->getPrato ()->getNmprato () 
			);
	}
	public static function exportaConfiguracaoCardapio($cdFilialOrigem, $cdServicoOrig, $dtInicialOrigem, $dtFinalOrigem, $cdFilialDestino, $cdServicoDestino, $dtInicialDestino, $dtFinalDestino, $configuracao = 'T', $idCopComensais = 'S', $idSobrepor, $cdoperador, $modoProcessamento, $nrlogintcard = null) {
		$procedure = new Procedure ( self::EXPORTA_CARDAPIO_CONFIG );
		$procedure->addParam ( new Param ( 'P_CDFILIORIG', Param::PARAM_INPUT, $cdFilialOrigem, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_CDSERVORIG', Param::PARAM_INPUT, $cdServicoOrig, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_DTINIORIG', Param::PARAM_INPUT, $dtInicialOrigem, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_DTFINORIG', Param::PARAM_INPUT, $dtFinalOrigem, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_CDFILIDEST', Param::PARAM_INPUT, $cdFilialDestino, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_CDSERVDEST', Param::PARAM_INPUT, $cdServicoDestino, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_DTINIDEST', Param::PARAM_INPUT, $dtInicialDestino, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_DTFINDEST', Param::PARAM_INPUT, $dtFinalDestino, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_CONFIGORIG', Param::PARAM_INPUT, $configuracao, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_IDCOPCOMENS', Param::PARAM_INPUT, $idCopComensais, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_IDSOBREPOR', Param::PARAM_INPUT, $idSobrepor, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_MODOPROC', Param::PARAM_INPUT, $modoProcessamento, Param::PARAM_TYPE_STR ) );
		$procedure->addParam ( new Param ( 'P_NRLOGINTCARD', Param::PARAM_OUTPUT, $nrlogintcard, Param::PARAM_TYPE_STR, 0 ) );
		
		$result = $procedure->call ();
		return $result;
	}
	public function deletaConfiguracaoMesSemCardapio($dtcardapio) {
		$manager = \Setforms2\Shared\Registry::getEntityManager ();
		
		$dtcardapio = '01/' . $dtcardapio;
		$dtfinal = DateUtil::getUltimoDiaDoMes ( DateUtil::getDataDeString ( $dtcardapio ) )->format ( DateUtil::FORMATO_BRASILEIRO );
		
		$this->cardadia->deletaCardapiosSemPrato ( $this->nrorg, $this->filial->getCdfilial (), $this->servico->getCdservfili (), $dtcardapio, $dtfinal );
		$manager->flush ();
		
		$this->deletaConfiguracaoMes ( $dtcardapio, $dtfinal );
		
		$manager->flush ();
		return true;
	}
	public function deletaConfiguracaoMes($dtInicial, $dtFinal) {
		$temCardapio = $this->cardadia->temCardapioNoMes ( $this->nrorg, $this->filial->getCdfilial (), $this->servico->getCdservfili (), $dtInicial, $dtFinal );
		
		if ($temCardapio) {
			return false;
		}
		
		/**
		 * @var $cnfcames \Planejamento\Repositories\Cnfcames
		 */
		$cnfcames = \Planejamento\Entities\Cnfcames::getRepository ();
		$cnfcames->deletaConfiguracaoDoMes ( $this->nrorg, $this->filial->getCdfilial (), $this->servico->getCdservfili (), $dtInicial );
	}
	public function deletaCardapioSemPrato($dtcardapio) {
		$dtfinal = $this->getUltimoDiaMes ( $dtcardapio );
		
		$this->cardadia->deletaCardapiosSemPrato ( $this->nrorg, $this->filial->getCdfilial (), $this->servico->getCdservfili (), $dtcardapio, $dtfinal );
	}
	public function deletaCardapioDoDia($dtcardapio) {
		$manager = \Setforms2\Shared\Registry::getEntityManager ();
		
		$cardapio = $this->getCardapioPorData ( $dtcardapio );
		if ($cardapio) {
			$cardapio->delete ();
		}
		
		$manager->flush ();
	}
	public function diasDisponiveisParaAlteracaoNoCardapio($dtcardapio, $dtprocessamento) {
		$dias = array ();
		
		$dtCardapio = new \TkDateTime ( $dtcardapio, 'dd/MM/y' );
		$dtprocessamento = new \TkDateTime ( $dtprocessamento->format ( 'd/m/Y' ), 'dd/MM/y' );
		
		$ultimoDiaDoMes = cal_days_in_month ( CAL_GREGORIAN, $dtCardapio->get ( 'M' ), $dtCardapio->get ( 'y' ) );
		
		$diferencaDasDatas = $dtCardapio->monthsAfter ( $dtprocessamento->get ( '01/MM/y' ) );
		$i = 1;
		if ($diferencaDasDatas === 0) {
			$i = $dtCardapio->daysAfter ( $dtprocessamento ) + 1;
		} else if ($diferencaDasDatas > 0) { // Nenhum dias pode ser alterado
			return false;
		}
		
		for(; $i <= $ultimoDiaDoMes; $i ++) {
			$dias [] = new \TkDateTime ( $i . '/' . $dtCardapio->get ( 'M' ) . '/' . $dtCardapio->get ( 'y' ), 'd/M/y' );
		}
		
		/**
		 * @var $dia \TkDateTime
		 */
		foreach ( $dias as $key => $dia ) {
//			$diaSemana = (strtoupper ( $dia->get ( 'e' ) ) != 6) ? strtoupper ( $dia->get ( 'EEE' ) ) : 'SAB';
//            if (in_array ( $diaSemana, $diasDaSemanaDesabilitados )) {
//                unset ( $dias [$key] );
//            } else
//            if (in_array ( 'FER', $diasDaSemanaDesabilitados ) && $this->isFeriado ( $dia->get ( 'dd/MM/y' ) )) {
            if ($this->isFeriado ( $dia->get ( 'dd/MM/y' ) )) {
				unset ( $dias [$key] );
			} else if ($dia->compare($dtprocessamento) === -1) {
				unset($dias[$key]);
			}
		}
		
		return $dias;
	}
	public function isFeriado($dtcardapio) {
		$ferifili = \Geral\Entities\Ferifili::getObjectByWhere ( 'CDFILIAL = ? AND DTFERIFILI = ?', array (
			$this->filial->getCdfilial (),
			$dtcardapio 
			) );
		return ( bool ) $ferifili;
	}
	public function getDescricaoPratosCardapio(\Planejamento\Entities\Cardadia $cardadia) {
		$descricao = array ();
		$compcardsEntity = $cardadia->getComposicoesDoCardapio ();
		$qttotal = $cardadia->getQtcomesticar ();
		/**
		 * @var $compcard \Planejamento\Entities\Compcard
		 */
		foreach ( $compcardsEntity as $compcard ) {
			$prato = $compcard->getPrato ();
			$descricao [$compcard->getNrordpratcar () - 1] ['VRPECONSCARD'] = round ( $compcard->getVrpeconscard (), 2 );
			$descricao [$compcard->getNrordpratcar () - 1] ['COMENSAIS'] = round ( $compcard->getVrpeconscard () / 100 * $qttotal, 2 );
			$descricao [$compcard->getNrordpratcar () - 1] ['NMPRATO'] = $prato->getNmprato ();
		}
		return $descricao;
	}
	public function getComposicaoPrato($nrsequencial, $dtcardapio) {
		if (empty ( $nrsequencial ) === false or $nrsequencial != 0) {
			$cardadia = $this->getCardapioPorData ( $dtcardapio );
			$compcard = $cardadia->getComposicaoDoCardapio ($nrsequencial);
			return $compcard;
		} else {
			return false;
		}
	}
	public static function retornaStatusDias($nrorg, $filial, $dtIni, $dtFin) {
		$dtInicial = DateUtil::getDataDeString ( $dtIni );
		$dtFinal = DateUtil::getDataDeString ( $dtFin );
		$dtProc = DateUtil::getDataDeString (\Estoque\Util\Parametros::getInstancia($filial)->getDataProcessamento());
		$intervaloDatas = DateUtil::getDatasEntre ( $dtInicial, $dtFinal );
		
		$datasBloq = array ();
		$datasPerm = array ();
		
		/**
		 * @var $data DateUtil
		 */
		foreach ( $intervaloDatas as $data ) {
            $foraPeriodoBloq = DateUtil::comparaDatas ( $data, $dtProc );
			$stData = DateUtil::truncateString ( $data );
			
			if (!$foraPeriodoBloq) {
                $datasPerm [] = $stData;
			} else {
				$ferifili = \Geral\Entities\Ferifili::getObjectByWhere ( 'CDFILIAL = ? AND DTFERIFILI = ?', array (
					$filial,
					$stData 
					) );
				
				if ($ferifili) {
					$datasBloq [] = $stData;
				} else {
					$datasPerm [] = $stData;
				}
			}
		}
		
		return array (
			'DATABLOQ' => $datasBloq,
			'DATAPERM' => $datasPerm 
			);
	}
	
	/**
	 * A partir deste ponto os metodos foram criados por
	 * 
	 * @author paulosoares
	 *         paulo.soares@teknisa.com
	 */
	public static function getCdfilialAuxPrat($nrorg, $cdprato) {
		if (empty ( $cdprato )) {
			return 'P';
		}
		/**
		 * @var $obj \Planejamento\Entities\Recepadr
		 */
		$obj = Recepadr::getObjectByWhere("NRORG = ? AND CDPRATO = ?",array($nrorg,$cdprato));
		 $cdfilialxprat = $obj->getCdfilauxprat();
		return $cdfilialxprat;
	}
	
	/**
	 * calcula o valor da aprova��o com base no campo de ganhp Coccao
	 */
	public static function calculaNovaAprovacaoCoccao($ganho) {
		if ($ganho == 0) {
			$novo_ganho = '100';
		} else {
			$novo_ganho = $ganho + 100;
		}
		
		return $novo_ganho;
	}
	
	/* calcula o valor do novo ganho de coccao com base na aprovacao */
	public static function calculaNovoGanho($aprov) {
		if ($aprov == '100') {
			return 0;
		} else {
			$n_aprov = $aprov - 100;
			return $n_aprov;
		}
	}
	
	/**
	 * calcula um nova aprova��o de coc��o com base no ganho
	 */
	public static function novaAprov($ganho) {
		if ($ganho == '0') {
			
			$novo_ganho = '100';
		} else {
			
			$novo_ganho = $ganho + 100;
		}
		return $novo_ganho;
	}
	
	/**
	 * Calculo de perda limpeza com base na aprova��o de limpeza
	 * 
	 * @param $value valor
	 *        	da aprova��o de limpeza
	 * @return number
	 */
	public static function calculaPerdaLimpeza($value) {
		return 100 - $value;
	}
	
	/**
	 * Calcula a aprova��o de limpeza com base na perda limpeza
	 * 
	 * @param $value valor
	 *        	da perda de limpeza
	 */
	public static function calculaNovaAprovacaoLimpeza($value) {
		return 100 - $value;
	}
	
	/**
	 * valida o campo de Aprovação Limpeza
	 * 
	 * @param unknown $valor        	
	 * @return boolean true se dentro do esperado.
	 */
	public static function validaAprovacaoLimpeza($valor) {
		if ($valor > '0' && $valor <= '100') {
			return true;
		}
		return false;
	}
	
	/**
	 * valida o campo de Perda Limpeza
	 * 
	 * @param unknown $valor        	
	 * @return boolean true se dentro do esperado
	 */
	public static function validaPerdaLimpeza($valor) {
		if ($valor >= '0' && $valor <= '99') {
			return true;
		}
		return false;
	}
	
	/**
	 * valida o campo de Aprovação Cocção
	 * se estiver na regra o retorno e true
	 */
	public static function validarAprovacaoCoccao($valor) {
		if ($valor > '0') {
			return true;
		}
		return false;
	}
	
	/**
	 * valida o campo de Ganho Cocção
	 * se estiver na regra o retorno e false
	 */
	public static function validarGanhoCoccao($valor) {
		if ($valor <= '-100') {
			return true;
		}
		return false;
	}
	
	/**
	 * valida o campo de Aprovação nutricional
	 * se estiver na regra o retorno e true
	 */
	public static function validarAprovacaoNutricional($valor) {
		if ($valor > 0 && $valor <= 100) {
			return true;
		}
		return false;
	}
	
	/**
	 * verifica se vai ser visivel ou não os campos de calculo.
	 */
	public static function validaVisibilidade($paramCampo) {
		if ($paramCampo == 'N') {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 *
	 * @param int $porcentagem 	valor da porcentagem a se acrescentar ou a decrementar do valor da quantidade
	 * @param int $quantidade    valor da quantidade
     *
	 * @return number valor da nova quantidade
	 */
	public static function calculaQuantidade($porcentagem, $quantidade) {
        $paramQuantidade = Parametros::getInstancia(tk_get_context_vars('CDFILIAL'))->getIdLiqBrut();

        if($paramQuantidade == 'L'){
            $novoValor = ((double)$quantidade * $porcentagem) / 100;
        } else{
            $novoValor = ($quantidade / $porcentagem) * 100;
        }

        return $novoValor;
	}
	public static function aprovLimpeza($novaAprov) {
		$novaPerda = '100' - $novaAprov;
		return $novaPerda;
	}
	public static function verificaCustoPadraoFechCard($nrorg, $filial, $servico, $mesCard, $custo, $dtBaseCard, $dtReferencia, $operador){
		if (self::getBloqFechCardapio() === 'S'){
			$datainicial = '01/'.$mesCard;
			$datafinal = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($datainicial))->format(DateUtil::FORMATO_BRASILEIRO);
			$stListaServ = $servico.'_';
			$idarquivo = 'T';
			$idprodutos = 'CFP';
			$idsepprato = 'T';
			$preacrescimo = 0;
			$idarredond = 'N';
			$cdprodutoini = 0;
			$cdprodutofin = 'zzzzzzzzzz';
			$nrconsplan = NovoCodigo::geraCodigo($nrorg, 'CONSPLAN');
			$stdtinicial = str_replace('/', '', '_'.DateUtil::getDataDeString($datainicial)->format(DateUtil::FORMATO_AMERICANO).'_');
			$stdtfinal = str_replace('/', '', '_'.DateUtil::getDataDeString($datafinal)->format(DateUtil::FORMATO_AMERICANO).'_');
				
			ConsumoPlanejadoRel::ProcedureConsPlan(
			$filial, $datainicial, $datafinal, $operador, $stListaServ, $idarquivo, $idprodutos, $idsepprato,
			$preacrescimo, $idarredond, $cdprodutoini, $cdprodutofin, $stdtinicial, $stdtfinal, $nrconsplan
			);
				
			/** @var $custPadrRepository Planejamento/Repositories/Custpadr */
			$custPadrRepository = Custpadr::getRepository();
			$difCusto = $custPadrRepository->verifDiferencaCustoPadrCard(
					$custo, $dtBaseCard, $dtReferencia, $nrconsplan, $filial, $servico, $datainicial, $datafinal
			);
				
			if (is_array($difCusto)) {
				$custoPrato = $difCusto['DIFECUSTO'];
				if ($custoPrato > 0) {
					if ($custoPrato > $difCusto['VRPEMAXDIFPLAPA']) {
						CardapioException::custoForaPeriodo();
					}
				}
	
				if ($custoPrato < 0) {
					if ($custoPrato > $difCusto['VRPEMINDIFPLAPA']) {
						CardapioException::custoForaPeriodo();
					}
				}
			}
		}
	}

    public static function updateCamposNotNullReceitaFilial($cdfiliaxprat,$cdprato,$cdfilial,$cdproduto,$nrorg){
        $connection = \SetForms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try{
            $receitasFilial = Recefili::getObjectsByWhere("NRORG = ? AND CDFILIAL = ? AND CDPRATO = ? AND
                                                        CDFILAUXPRAT = ? AND CDPRODUTO = ?",
                                                        array($nrorg,$cdfilial,$cdprato,$cdfiliaxprat,$cdproduto));

            /** @var $receitaFilial \Planejamento\Entities\Recefili */
           foreach($receitasFilial as $receitaFilial){
                if($receitaFilial->getIdativo() === null){
                    $receitaFilial->setIdativo('S');
                }
               if($receitaFilial->getIdutcompentdirf() === null){
                   $receitaFilial->setIdutcompentdirf('N');
               }

               $receitaFilial->setDtultatu(DateUtil::getDataAtual());
               $receitaFilial->setCdoperultatu(Ambiente::getCdOperador());
               $receitaFilial->setNrorgultatu($nrorg);

               $receitaFilial->persist();
           }
            $connection->flush();
            $connection->commit();
        }catch (\Exception $e){
            tk_warning($e->getMessage());
            $connection->rollback();
            $connection->clear();
        }
    }
   public  static function updateCamposNotNullReceitaServico($cdfiliaxprat,$cdprato,$cdfilial,$cdproduto,$nrorg){
       $connection = \SetForms2\Shared\Registry::getEntityManager();
       $connection->beginTransaction();
       try{
           /** @var $receitasServico \Planejamento\Entities\Receserv */
           $receitasServico = Receserv::getObjectsByWhere("NRORG = ? AND CDFILIAL = ? AND CDPRATO = ? AND
                                                        CDFILAUXPRAT = ? AND CDPRODUTO = ?",
                                                        array($nrorg,$cdfilial,$cdprato,$cdfiliaxprat,$cdproduto));
           /** @var $receitaServico \Planejamento\Entities\Receserv*/
           foreach($receitasServico as $receitaServico){
               if($receitasServico->getIdativo() ==  null){
                   $receitaServico->setIdativo('S');
               }
               if($receitaServico->getIdutcompentdirs() == null){
                   $receitaServico->setIdutcompentdirs('N');
               }

               $receitaServico->setDtultatu(DateUtil::getDataAtual());
               $receitaServico->setCdoperultatu(Ambiente::getCdOperador());
               $receitaServico->setNrorgultatu($nrorg);

               $receitaServico->persist();
           }
           $connection->flush();
           $connection->commit();
       }catch (\Exception $e){
           tk_warning($e->getMessage());
           $connection->rollback();
       }

    }

    public static function updateCamposNotNullReceitaPadrao($cdfiliaxprat,$cdprato,$cdfilial,$cdproduto,$nrorg){
		$connection = Registry::getEntityManager();
        $connection->beginTransaction();
        try{
            tk_warning($cdfiliaxprat);
            /** @var $receitasServico \Planejamento\Entities\Recepadr */
            $receitasServico = Recepadr::getObjectByWhere("NRORG = ? AND CDPRATO = ? AND
                                        	                CDFILAUXPRAT = ? AND CDPRODUTO = ?",
										    	            array((string)$nrorg,$cdprato,$cdfiliaxprat,$cdproduto));

            /** @var $receitaServico \Planejamento\Entities\Recepadr */
			foreach ($receitasServico as $receitaServico){
                if($receitaServico->getIdativo() == null){
                    $receitaServico->setIdativo('S');
                }
                if($receitaServico->getIdutcompentdirp() == null){
                	$receitaServico->setIdtpreqprodpadr('N');
                }

                $receitaServico->setDtultatu(DateUtil::getDataAtual());
                $receitaServico->setCdoperultatu(Ambiente::getCdOperador());
                $receitaServico->setNrorgultatu($nrorg);

                $receitaServico->persist();
            }
            $connection->flush();
            $connection->commit();
        }catch (\Exception $e){
            tk_warning($e->getMessage());
            $connection->rollback();
        }
    }

    /**
     * retrona se os campos de calculo da revisão da receita
     * @static
     * @param string $nrorg número da organização
     * @param string $cdfilial código da filial
     * @return array
     */
    public static function getParametrosVisibilidade($nrorg,$cdfilial){
        /** @var $paramplaneja \Planejamento\Entities\Paramplaneja */
        $paramplaneja = self::getEntityParamPlaneja($nrorg,$cdfilial);
        $ganho = $paramplaneja->getIdganhococc();
        $perda = $paramplaneja->getIdperdalimp();
        $aprovacaoCoccao = $paramplaneja->getIdaprovcocc();
        $aprovacaoLimpeza = $paramplaneja->getIdaprovlimp();
        $tempoPreparo = $paramplaneja->getIdtempprep();
        $visibilidadeCampos  = array($ganho, $perda,$aprovacaoCoccao,$aprovacaoLimpeza,$tempoPreparo);
        return $visibilidadeCampos;
    }

    /**
     * Recebe um array com diversos servicos e retorna uma string
     * com um servico de cada separados por '_'
     * @param array $servicos
     * @return string $listaServico
     */
    public static function retornaListaServico(array $servicos){
        $listaServico = '_';
        $cdServicos = array();

        foreach($servicos as $servico){
            array_push($cdServicos,$servico['CDSERVFILI']);
        }
        $cdServicos = array_unique($cdServicos);
        foreach($cdServicos as $cdServico){
            $listaServico .= $cdServico.'_';
        }

        return $listaServico;
    }

    public static function retornaListaFilialGrupfiliContrato($filial,$cdgrupfili,$nrorg){
        $listaFilial = '';
        if ($filial == 'A') {
            if (empty($cdgrupfili)) {
                $arrayFilial = Filial::getObjectsByWhere(
                    "NRORG = ? AND IDCNTRRESC <> 'S'",
                    array($nrorg)
                );
                foreach($arrayFilial as $filial){
                    $listaFilial .= $filial->getCdfilial().'_';
                }
            } else {
                $filial = \Geral\Entities\Filial::getRepository();
                $arrayFilial = $filial->consultaFilialPorGrupoContrato($cdgrupfili, $nrorg);
                foreach($arrayFilial as $filial){
                    $listaFilial .= $filial['CDFILIAL'].'_';
                }
            }
        } else{
            if(is_array($filial)){
                foreach($filial as $fili){
                    $listaFilial .= $fili['cdfilial'].'_';
                }
            } else{
                $listaFilial = $filial;
            }
        }

        return $listaFilial;
    }

    public static function tratamentoDeParaServico($cdfilialori,$cdfilialdest,$listaFilialDest,$cdgrupfili,$cdservori,$tpServHosp,$tpServAmb,$tpServRes,$copiarFiliais,$nrorg){
        $servicoRepositorio = \Geral\Entities\Servico::getRepository();
        $arrayAux = array();

        if($cdfilialdest == $cdfilialori){
            $listaServico = $servicoRepositorio->retornaDeParaServicoMesmaFilial($cdfilialori,$cdservori,$tpServHosp,$tpServAmb,$tpServRes,$nrorg);
        } else{
            if($copiarFiliais == 'S'){
                $listaFilialDest = self::retornaListaFilialGrupfiliContrato($listaFilialDest,$cdgrupfili,$nrorg);
                $listaServico = $servicoRepositorio->retornaDeParaServicoTodasFiliais($cdfilialori,$listaFilialDest,$cdservori,$tpServHosp,$tpServAmb,$tpServRes,$nrorg);
            } else{
                $listaServico = $servicoRepositorio->retornaDeParaServicoFilialDiferente($cdfilialori,$cdfilialdest,$cdservori,$tpServHosp,$tpServAmb,$tpServRes,$nrorg);
            }
        }

        foreach($listaServico as $indice=>$servico){
            if($servico['CDSERVORIG'] == $servico['CDSERVDEST']){
                array_push($arrayAux,$servico['CDSERVDEST']);
            } else{
                if(in_array($servico['CDSERVDEST'],$arrayAux)){
                    unset($listaServico[$indice]);
                }
            }
        }

        $arrayAux = array();

        foreach($listaServico as $indice=>$servico){
            if(!empty($servico['CDSERVDEST'])){
                if(in_array($servico['CDSERVDEST'],$arrayAux)){
                    unset($listaServico[$indice]);
                } else{
                    array_push($arrayAux,$servico['CDSERVDEST']);
                }
            }
        }

        return $listaServico;
    }

    //PARAMETROS

    /**
     * pega o objeto paramplaneja
     * @static
     * @param string $nrorg número da organização
     * @param string $cdfilial código da filial
     * @return object paramplaneja
     */
    public static function getEntityParamPlaneja($nrorg,$cdfilial){
        $paramplaneja = Paramplaneja::getObjectByWhere('NRORG = ? AND CDFILIAL  = ?',array($nrorg,$cdfilial));
        return $paramplaneja;
    }

    /**
     * pega o objeto paramgeral
     * @static
     * @param string $nrorg número da organização
     * @param string $cdfilial código da filial
     * @return object paramgeral
     */
    public static function entidadeParamGeral($nrorg){
        $paramgeral = Paramgeral::getObjectByWhere('NRORG = ?',array($nrorg));
        tk_set_env_var('PARAMGERAL',$paramgeral);

        return $paramgeral;
    }

    /**
     * pega o objeto paramfilial
     * @static
     * @param string $nrorg número da organização
     * @param string $cdfilial código da filial
     * @return object paramfilial
     */
    public static function entidadeParamFilial($nrorg,$cdfilial){
        $paramfilial = Paramfilial::getObjectByWhere('NRORG = ? AND CDFILIAL  = ?',array($nrorg,$cdfilial));
        tk_set_env_var('PARAMFILIAL', $paramfilial);

        return $paramfilial;
    }

    public static function entidadeParamPlaneja($nrorg,$cdfilial){
        $paramplaneja = Paramplaneja::getObjectByWhere('NRORG = ? AND CDFILIAL  = ?',array($nrorg,$cdfilial));
        tk_set_env_var('PARAMPLANEJA', $paramplaneja);

        return $paramplaneja;
    }

    public static function getNrCorEntrsaf(){
        $param = tk_get_env_var('PARAMPLANEJA');
        $cor = $param->getNrcorentrsaf();
        $corHex = dechex($cor);
        $corHex = str_pad($corHex, 6 , 0, STR_PAD_LEFT);
        return '#'.$corHex;
    }

    public static function getNrCorInicSaf() {
        $param = tk_get_env_var('PARAMPLANEJA');
        $cor = $param->getNrcorinicsaf();
        $corHex = dechex($cor);
        $corHex = str_pad($corHex, 6 , 0, STR_PAD_LEFT);
        return '#'.$corHex;
    }

    public static function getNrCorSafra() {
        $param = tk_get_env_var('PARAMPLANEJA');
        $cor = $param->getNrcorsafra();
        $corHex = dechex($cor);
        $corHex = str_pad($corHex, 6 , 0, STR_PAD_LEFT);
        return '#'.$corHex;
    }

    public static function getCorDiaNaoUtil() {
        $param = tk_get_env_var('PARAMPLANEJA');
        $cor = $param->getNrcornutil();
        $corHex = dechex($cor);
        $count = strlen($corHex);
        if(strlen($corHex) == 2) {
            $corHex = str_pad($corHex, 6 , 0);
        }
        return '#'.$corHex;
    }

    public static function getCorBloqAlt() {
        $param = tk_get_env_var('PARAMPLANEJA');
        $cor = $param->getNrcornalte();
        $corHex = dechex($cor);
        $count = strlen($corHex);
        if(strlen($corHex) == 2) {
            $corHex = str_pad($corHex, 6 , 0);
        }
        return '#'.$corHex;
    }

    public static function getCorCardEspecifico() {
        $param = tk_get_env_var('PARAMPLANEJA');
        $cor = $param->getNrcorcardesp();
        $corHex = dechex($cor);
        return '#'.$corHex;
    }

    public static function getBloqFechCardapio() {
        $param = tk_get_env_var('PARAMPLANEJA');
        $bloqFech = $param->getIdblqfechcard();
        return empty($bloqFech) === true ? 'N' : $bloqFech;
    }

}